require 'net/ssh/script/version'
require 'net/ssh/script/command'

module Net
  module SSH
    module Script
      # Net::SSH::Command wrapper to work with multiple commands.
      class Script
        def self.[](commands)
          new commands
        end

        def initialize(commands)
          if commands.is_a? String
            @_commands = [Command[commands]]
          elsif commands.is_a?(Array) &&
                commands.all? { |cmd| cmd.is_a? String }
            @_commands = commands.map { |c| Command[c] }
          elsif commands.is_a?(Array) &&
                commands.all? { |cmd| cmd.is_a? Command }
            @_commands = commands
          else raise 'Invalid script argument.'
          end
        end

        def run(ssh)
          @_commands.each { |cmd| break unless cmd.run(ssh) }
          ok?
        end

        def done?
          @_commands.all?(&:done?)
        end

        def status
          return :pending if @_commands.all?(&:pending?)
          return :failed if @_commands.any?(&:failed?)
          return :error if @_commands.any?(&:error?)
          return :running unless done?
          :ok
        end

        def to_s
          @_commands.map(&:to_s).join "\n"
        end

        def inspect
          @_commands.map(&:inspect).join "\n"
        end

        [:pending?, :running?, :failed?, :error?, :ok?].each do |helper|
          define_method helper do
            status == helper.to_s.sub('?', '').to_sym
          end
        end
      end
    end
  end
end
